#!/usr/bin/perl -w
use strict;
use ExtUtils::MakeMaker;
use File::Spec;
use CTK::Util qw/sharedstatedir sharedir/;
use File::Spec qw//;
use MPMinus::Helper::Util qw/getApache/;

use constant {
        PROJECT_NAME  => "Kinoteka",
        PROJECT_NAMEL => "kinoteka",
    };

my $httpd_root = getApache("HTTPD_ROOT");
my $config_dir;
for (qw/vhosts.d sites-available conf.d/) {
    $config_dir = File::Spec->catdir($httpd_root, $_);
    last if -e $config_dir;
}
$config_dir = File::Spec->tmpdir unless $config_dir && -e $config_dir;

my $build_requires = {
        'CTK'                   => 2.03,
        'ExtUtils::MakeMaker'   => 6.60,
        'MPMinus'               => 1.21,
        'Test::More'            => 0.94,
    };

my $prereq_pm = {
        'CGI'                   => 4.00, # perl-CGI
        'CTK'                   => 2.03, # perl-CTK
        'DBD::SQLite'           => 1.00, # perl-DBD-SQLite (libdbd-sqlite3-perl)
        'Encode::Locale'        => 1.00, # perl-Encode-Locale
        'File::Copy'            => 2.09,
        'List::Util'            => 1.50, # (libscalar-list-utils-perl)
        'LWP::UserAgent'        => 6.15,
        'MPMinus'               => 1.21,
        'URI'                   => 1.73,
    };

WriteMakefile(
    'NAME'              => PROJECT_NAME,
    'DISTNAME'          => PROJECT_NAMEL,
    'MIN_PERL_VERSION'  => 5.016001,
    'VERSION_FROM'      => 'lib/Kinoteka.pm',
    'ABSTRACT'          => 'Kinoteka - Minus\'s Film Library',
    'BUILD_REQUIRES'    => $build_requires,
    'PREREQ_PM'         => $prereq_pm,
    'AUTHOR'            => 'Serz Minus (Sergey Lepenkov) <minus@serzik.com>',
    'LICENSE'           => 'GPL',
    'META_MERGE'        => {
        resources => {
            homepage        => 'https://www.serzik.com',
            repository      => 'https://www.serzik.com:8008/misc/kinoteka',
            license         => 'http://opensource.org/licenses/gpl-license.php',
        },
    },
    macro => {
            PROJECT_NAME    => PROJECT_NAME,
            PROJECT_NAMEL   => PROJECT_NAMEL,
            PROJECT_CONFDIR => '$(DESTDIR)' . $config_dir,
            PROJECT_DATADIR => '$(DESTDIR)' . File::Spec->catdir(sharedstatedir(), PROJECT_NAMEL),
            PROJECT_SHAREDIR => '$(DESTDIR)' . File::Spec->catdir(sharedir(), PROJECT_NAMEL),
        },
    clean => {
            FILES => '$(INST_CONF) *.tmp',
        },
);

1;

package MY;
use CTK::Util qw/dformat/;

sub postamble {
dformat(<<'MAKE_FRAG', {TAB => "\t"});
INST_CONF = blib$(DFSEP)conf
INST_SHARE = blib$(DFSEP)share

pure_all :: configured.tmp
[TAB]$(NOECHO) $(ECHO) "Configured."

# Configure
configured.tmp : src$(DFSEP)$(PROJECT_NAMEL).conf $(INST_CONF)$(DFSEP).exists $(INST_SHARE)$(DFSEP).exists
[TAB]$(CP) src$(DFSEP)$(PROJECT_NAMEL).conf $(INST_CONF)
[TAB]$(CP) -R www$(DFSEP)* $(INST_SHARE)$(DFSEP)
[TAB]$(NOECHO) $(TOUCH) configured.tmp

# Create config directory
$(INST_CONF)$(DFSEP).exists :: Makefile.PL
[TAB]$(MKPATH) $(INST_CONF)
[TAB]$(NOECHO) $(CHMOD) $(PERM_DIR) $(INST_CONF)
[TAB]$(NOECHO) $(TOUCH) $(INST_CONF)$(DFSEP).exists

# Create share directory
$(INST_SHARE)$(DFSEP).exists :: Makefile.PL
[TAB]$(MKPATH) $(INST_SHARE)
[TAB]$(NOECHO) $(CHMOD) $(PERM_DIR) $(INST_SHARE)
[TAB]$(NOECHO) $(TOUCH) $(INST_SHARE)$(DFSEP).exists

install :: installed.tmp
[TAB]$(NOECHO) $(ECHO) The $(PROJECT_NAME) project has been successfully installed.

installed.tmp : configured.tmp
[TAB]$(MKPATH) $(PROJECT_CONFDIR)
[TAB]$(NOECHO) $(CHMOD) $(PERM_DIR) $(PROJECT_CONFDIR)
[TAB]$(TEST_F) $(PROJECT_CONFDIR)$(DFSEP)$(PROJECT_NAMEL).conf && $(CP) $(INST_CONF)$(DFSEP)$(PROJECT_NAMEL).conf $(PROJECT_CONFDIR)$(DFSEP)$(PROJECT_NAMEL).conf.orig || $(CP) $(INST_CONF)$(DFSEP)$(PROJECT_NAMEL).conf $(PROJECT_CONFDIR)$(DFSEP)$(PROJECT_NAMEL).conf
[TAB]$(MKPATH) $(PROJECT_DATADIR)
[TAB]$(NOECHO) $(CHMOD) $(PERM_DIR) $(PROJECT_DATADIR)
[TAB]$(MKPATH) $(PROJECT_SHAREDIR)
[TAB]$(NOECHO) $(CHMOD) $(PERM_DIR) $(PROJECT_SHAREDIR)
[TAB]$(CP) -R $(INST_SHARE)$(DFSEP)* $(PROJECT_SHAREDIR)$(DFSEP)
[TAB]$(NOECHO) $(TOUCH) installed.tmp
MAKE_FRAG
}

1;

__END__
