/*  $Id: app.js 17 2021-03-12 18:41:38Z minus $
 *
 *  app.js
 *  Version 1.00 2021
 *  Kinoteka Application
 *                                                       ____________
 *  Copyright (C) 1998-2021 D&D Corporation.             \          /
 *  All rights reserved.                                  \________/
 *  Serż Minus (Sergey Lepenkov), http://www.serzik.com    \      /
 *                                                          \____/
 *  Dual licensed under the MIT and GNU GPL v3. licenses:    \  /
 *   http://www.opensource.org/licenses/mit-license.php       \/
 *   http://www.gnu.org/licenses/gpl.html
 *
*/
"use strict";


/*
    Radio
*/
const channel = Backbone.Radio.channel(PROJECT_PREFIX);


/*
    Objects
*/
const MyObject = Marionette.MnObject.extend({
    model:      {},
    collection: {},
    view:       {},
    controller: {},
    app:        {},
    timer:      {},
    cntLoading:         0, // For loading bar
    sitedata_is_loaded: false,
    error_message: "",

    initialize() {
        var self = this;
        this.setYear("????");
        //this.search = $.getQSVar('search') || "";
        //this.page = ($.getQSVar('page') || 0) * 1;
        //this.id = ($.getQSVar('id') || 0) * 1;
    },
    ajaxErrorHandler(jqXHR, jx, url) {
        jx.status_code = jqXHR.status;
        jx.status_text = jqXHR.statusText;
        var def_msg = "Can't get data from " + (url || 'server');
        if (jqXHR.responseJSON) {
            jx.message = jqXHR.responseJSON.error || jqXHR.responseJSON.message || def_msg;
            //console.log(jx.message);
        } else {
            jx.status_code = jqXHR.status;
            jx.status_text = jqXHR.statusText;
            jx.message = def_msg;
        }
        this.setMessage(jx.message);
        console.log(jqXHR.statusText);
    },
    alert(ajx) {
        this.pushAlert({
            noAdd: true,
            level: "danger",
            code: ajx.status_code,
            title: ajx.status_text,
            message: ajx.message,
        });
    },
    pushAlert(data) { // Show alert message
        var tpl = _.template($('#tplAlert').html());
        var html = tpl({
            level: data.level || "primary",
            code: data.code || 0,
            title: data.title || "",
            message: data.message || "",
        });
        if (data.noAdd) {
            $('#status-bar').html(html);
        } else {
            $('#status-bar').append(html);
        }
        return true;
    },
    cleanAlert() { // Delete all alerts
        $('#status-bar').empty();
    },
    showLoading() { // Show loading bar
        this.cntLoading++;
        $('#loading').show();
    },
    hideLoading() { // Hide loading bar
        this.cntLoading--;
        if (this.cntLoading <= 0) {
            $('#loading').hide();
        }
    },
    setTitle(t) {
        // Change Title & Description
        var title = t || PROJECT_VERSION;
        document.title = PROJECT_NAME + " · " + title;
        $('meta[name="description"]').attr("content", PROJECT_NAME + " · " + title);
    },
    setYear(year) {
        $('#year').text(year);
    },
    setMessage(msg) {
        this.error_message = msg;
    },
    setTimer() {
        var self = this;
        //this.search = $.getQSVar('search') || "";
        //this.page = ($.getQSVar('page') || 0) * 1;
        //this.id = ($.getQSVar('id') || 0) * 1;

        // Set timer
        this.timer.minutely = setInterval(function() {
            //self.app.graphsApp.triggerMethod("reload", "Run app trigger reload from MyObject inicialize()");
            //self.reload("Run minutelyReload() from MyObject inicialize()");
            //console.log("Запустился минутный таймер! Поменяй меня - я должен запускаться раз в 60 секунд :-)");
            channel.trigger("all:reload", "Run channel trigger all:reload from MyObject setTimer()");
        }, 60*1000); // Every 1 minute
    },
    load_sitedata() { // Load sitedata
        var self = this;
        var model = this.model.sitedata;
        var url = model.url;

        // Get data
        var ajx = Object.assign({}, ajaxDefaults);
        var defer = $.Deferred();
        self.showLoading();
        model.fetch({
            timeout: JQX_TIMEOUT,
            cache: true,
            success: function(mdl){
                defer.resolve(mdl);
            },
            error: function(mdl, jqXHR, o){
                self.ajaxErrorHandler(jqXHR, ajx, url);
                defer.resolve(undefined);
            },
        });
        $.when(defer.promise()).done(function(mdl){
            if (mdl !== undefined) { // OK
                if (self.sitedata_is_loaded == false) {
                    self.sitedata_is_loaded = true;
                    self.cleanAlert();
                }
                //console.log("Sitedata is loaded")
                //console.log(mdl);
                //var settings = mdl.get("settings");
                //if (_.isObject(settings)) {
                //    self.settings = settings;
                //}
                //self.load_meta();

                self.setTitle("v" + mdl.get("version"));
                self.setYear(mdl.get("year"));

                channel.trigger("all:ready", "Run channel trigger all:ready from MyObject load_sitedata()");
                //self.setTimer(); // For regular reloading
            } else {
                self.sitedata_is_loaded = false;
                self.alert(ajx);
            }
            self.hideLoading();
        });
    },
    run() {
        this.load_sitedata();
    },
});
const kino = new MyObject();

/*
    Controllers (Behaviors)
*/
kino.controller.boxFilmAddShow = Marionette.Behavior.extend({ // Button/Link to show of film add box
    ui: {
        btnFilmAdd: ".btnFilmAdd",
    },
    events: {
        'click @ui.btnFilmAdd': "onClickBtnFilmAdd",
    },
    onClickBtnFilmAdd(e) {
        e.preventDefault();
        e.stopPropagation();
        //this.view.trigger("boxfilmadd:show", $(e.target).attr("data-id")); // Go to next
        //console.log($(e.target).attr("data-id"));
        this.view.trigger("boxfilmadd:show", $(e.currentTarget).attr("data-id") > 0 ? this.view.model : null); // Go to next 
    },
});
kino.controller.boxFilmViewShow = Marionette.Behavior.extend({ // Button/Link to show of film box
    ui: {
        btnFilmView: ".btnFilmView",
    },
    events: {
        'click @ui.btnFilmView': "onClickBtnFilmView",
    },
    onClickBtnFilmView(e) {
        e.preventDefault();
        e.stopPropagation();
        this.view.trigger("boxfilmview:show", this.view.model); // Go to next 
    },
});



/*
    Models & Collections
*/
const SitedataModel = Backbone.Model.extend({
    url: SITEDATA_URL,
    defaults: {
        "server_status":0,
        "version":      "unknown",
        "year":         "2021",
        "sid":          "",
        "message":      "",
        "debug":        "off",
        "time":         "",
        "date":         "",
        "remote_addr":  "",
        "total":        0,
        "storage_last": 0, // kino.model.sitedata.get("storage_last")
        "genres":       [],
        "countries":    [],
    },
});
kino.model.sitedata = new SitedataModel();

// Films model
const FilmsModel = Backbone.Model.extend({
    url: JQX_PFX_URL+"/films",
    defaults: {
        "last":         [], // list of last films
        "random":       [], // list of random films
    },
});
kino.model.films = new FilmsModel();

// Film model & Films collection
const FilmModel = Backbone.Model.extend({
    url: JQX_PFX_URL+"/film",
    defaults: {
        "id":           0,
        "obsolete":     null,
        "storage":      null,
        "name":         "",
        "nameorig":     "",
        "genre":        "",
        "date":         null,
        "imdbid":       "",
        "kinopoiskid":  "",
        "country":      "",
        "director":     "",
        "scriptwriter": "",
        "producer":     "",
        "operator":     "",
        "music":        "",
        "artdirector":  "",
        "cut":          "",
        "time":         "",
        "ratingimdb":   "",
        "ratingkinopoisk": "",
        "description":  "",
        "packshot":     "",
    },
});
kino.model.film = new FilmModel();
const FilmsCollection = Backbone.Collection.extend({
    model: FilmModel,
});

// Search model
const SearchModel = Backbone.Model.extend({
    url: JQX_PFX_URL + "/search",
    defaults: {
        "message":  "",
        "id":       0,
        "name":     '',
        "storage":  '',
        "genre":    '',
        "country":  '',
        "date":     '',
        // Result
        "total":    0,
        "page":     0,
        "pages":    0,
        "films":    [], // list of films (result)
        "rows_per_page": 0,
    },
});
kino.model.search = new SearchModel();

// Pagination model
const PaginationModel = Backbone.Model.extend({
    defaults: {
        "page":         0,
        "title":        "",
        "label":        "",
        "is_active":    0,
        "is_disabled":  0,
      },
});
const PaginationCollection = Backbone.Collection.extend({
    model: PaginationModel,
});
kino.collection.pagination = new PaginationCollection();

/*
    Common views
*/
kino.view.empty = Marionette.View.extend({ // kino.view.empty({message: msg});
    tagName: "span",
    className: "text-muted p-3",
    template: _.template('<%= message %>'),
    initialize: function(options){
        var opts = options || {};
        this.message = opts.message || "No data";
    },
    serializeData: function(){
        return {
            message: this.message,
        };
    },
});
kino.view.emptyDiv = Marionette.View.extend({ // kino.view.emptyDiv({message: msg});
    tagName: "div",
    template: _.template('<p><%= message %></p>'),
    initialize: function(opts){
        this.options.message = opts.message || "No data";
    },
    serializeData: function(){
        return {
            message: this.options.message,
        };
    },
});
kino.view.alert = Marionette.View.extend({
    template: _.template($('#tplAlert').html()),
    initialize: function(options){
        var opts = options || {};
        this.level  = opts.level || "danger"; // primary, secondary, success, danger, warning, info, light, dark  # https://getbootstrap.com/docs/4.1/components/alerts/ 
        this.code   = opts.code || 0;
        this.title  = opts.title || "Error!";
        this.message= opts.message || "unknown error";
    },
    serializeData: function(){
        var code = this.code;
        var msg  = this.message;
        if (!code && msg == "timeout") {
            code = "-408";
            msg = "Timed out!";
        } else if (!code && msg == "error") {
            code = "-400";
            msg = "Can't connect to server!";
        }
        return {
            level: this.level,
            title: this.title,
            code: code,
            message: msg,
        };
    },
});
kino.view.btnSend = Marionette.View.extend({
    tagName: "button",
    className: "btn btn-primary",
    template: _.template('<%= message %>'),
    attributes: {
        type: "button",
    },
    initialize(message, type) {
        var self = this;
        this.message = message || "Send";
        if (type && type == "submit") {
            this.$el.attr("type", type);
        } else {
            this.$el.on("click", function(e){ self.triggerMethod("button:send:click", self, e); });
        }
    },
    templateContext() {
        return {
            message: this.message,
        };
    },
});
kino.view.btnLoading = Marionette.View.extend({
    tagName: "button",
    className: "btn btn-primary",
    template: _.template('<span class="spinner-border spinner-border-sm" role="status" aria-hidden="true"></span>&nbsp;<%= message %>'),
    attributes: {
        disabled: 1,
        type: "button",
    },
    initialize(message) {
        this.message = message || "Sending...";
    },
    templateContext() {
        return {
            message: this.message,
        };
    },
});
kino.view.button = Marionette.View.extend({
    tagName: "a",
    className: "btn",
    template: _.template('<%= caption %>'),
    attributes: {
        role: "button",
    },
    initialize() {
        var self = this;
        this.$el.attr("href", self.model.get("href"));
        var cclass = self.model.get("class");
        if (! _.isEmpty(cclass)) { 
            this.$el.addClass(cclass);
        }
    },
});
kino.view.buttons = Marionette.CollectionView.extend({
    tagName: "span",
    childView: kino.view.button,
});

// Modals
kino.view.boxFilmAdd = Marionette.View.extend({
    template: _.template($('#tplBoxFilmAdd').html()),
    ui: {
        modalBoxFilmAdd:    "#modalBoxFilmAdd",
        formFilmAdd:        "#formFilmAdd",
        fldFilmGenre:       "#fldFilmGenre",
        fldFilmGenreSelect: "#fldFilmGenreSelect",
        fldFilmCountry:     "#fldFilmCountry",
        fldFilmCountrySelect: "#fldFilmCountrySelect",
        fldFilmPackshot:    "#fldFilmPackshot",
        fldFilmStorage:     "#fldFilmStorage",
        fldFilmName:        "#fldFilmName",
        btnBoxFilmAdd:      "#btnBoxFilmAdd",
    },
    templateContext() {
        // Genre and Country lists
        var genres = [];
        var countries = [];
        if (_.isObject(kino.model.sitedata)) {
            genres = kino.model.sitedata.get("genres");
            countries = kino.model.sitedata.get("countries");
        }

        return {
            "storage": this.model.get("storage") > 0 ? this.model.get("storage") : kino.model.sitedata.get("storage_last"),
            "genres": _.isEmpty(genres) ? [] : genres,
            "countries": _.isEmpty(countries) ? [] : countries,
        };
    },
    events: {
        'click @ui.btnBoxFilmAdd': 'onClickBtnFilmAdd',
        'submit @ui.formFilmAdd': 'onSubmitFormFilmAdd',
        'change @ui.fldFilmGenreSelect': 'onChangeFilmGenre',
        'change @ui.fldFilmCountrySelect': 'onChangeFilmCountry',
    },
    onClickBtnFilmAdd(e) {
        this.ui.formFilmAdd.submit();
    },
    onSubmitFormFilmAdd(e) {
        e.preventDefault();
        e.stopPropagation();

        // Validity;
        if (this.ui.formFilmAdd[0].checkValidity() === false) {
            this.ui.formFilmAdd[0].reportValidity();
            return false;
        }

        // Storage check
        var ui_storage = this.ui.fldFilmStorage;
        if (ui_storage.val() * 1 > 0) { // Ok
            $(ui_storage).removeClass("is-invalid").addClass("is-valid");
        } else { // Error
            $(ui_storage).removeClass("is-valid").addClass("is-invalid");
            return false;
        }

        // Name check
        //var ui_name = this.ui.fldFilmName;
        //if (_.isString(ui_name.val()) && ui_name.val().length > 0) { // Ok
        //    $(ui_name).removeClass("is-invalid").addClass("is-valid");
        //} else { // Error
        //    $(ui_name).removeClass("is-valid").addClass("is-invalid");
        //    return false;
        //}


        // Create form data object
        var formdata = new FormData();
        var files = this.ui.fldFilmPackshot.prop('files');
        if (!(typeof(files) == 'undefined')) {
            var file = files[0];
            if (!(typeof(file) == 'undefined')) {
                formdata.append("packshot", file); // Add first file from files array!!
                formdata.append("size", file.size);
            }
        }
        var sa = this.ui.formFilmAdd.serializeArray();
        $.each(sa, function(){
            formdata.append(this.name, this.value);
        });
        formdata.append("id", this.model.get("id"));

        // Disable button
        this.ui.btnBoxFilmAdd.prop("disabled", "disabled");

        // Go to Root form
        this.trigger("filmadd:submit", this, formdata);

        //this.ui.modalBoxFilmAdd.modal('hide');
        return false;
    },
    onChangeFilmGenre(e) {
        var target = e.target;
        var genre = $(target).val(); // target.options[target.selectedIndex].value
        this.ui.fldFilmGenre.val(genre);
        //console.log( genre );
    },
    onChangeFilmCountry(e) {
        var target = e.target;
        var country = $(target).val();
        this.ui.fldFilmCountry.val(country);
    },
    progress(v) {
        this.ui.btnBoxFilmAdd.text(v+"%");
    },
    hideModal() {
        this.ui.modalBoxFilmAdd.modal('hide');
    },
    showModal(mdl) {
        if (_.isObject(mdl)) {
            //console.log(mdl);
            this.model = mdl;
        } else {
            //console.log(this.model);
            this.model = kino.model.film;
            this.model.clear;
        }
        this.render();
        this.ui.modalBoxFilmAdd.modal('show');
    },
});
kino.view.boxFilmView = Marionette.View.extend({
    template: _.template($('#tplBoxFilmView').html()),
    ui: {
        modalBoxFilmView:   "#modalBoxFilmView",
        btnSearchByID:      "#btnSearchByID",
    },
    events: {
        'click @ui.btnSearchByID': 'onClickBtnSearchByID',
    },
    onClickBtnSearchByID(e) {
        e.preventDefault();
        e.stopPropagation();

        // Go to Root form
        this.trigger("searchbyid:click", this, this.model.id);

        return false;
    },
    hideModal() {
        this.ui.modalBoxFilmView.modal('hide');
    },
    showModal(mdl) {
        this.model = mdl;
        this.render();
        this.ui.modalBoxFilmView.modal('show');
        $('.lightzoom').lightzoom({
            viewTitle: true,
            isOverlayClickClosing: true,
            isWindowClickClosing: true,
            isEscClosing: true,
        });
    },
});

// Section: Home
kino.view.itemLast = Marionette.View.extend({
    tagName: "div",
    className: "p-1",
    template: _.template($('#tplItemLast').html()),
    templateContext() {
        return {
            title: cutLongString(this.model.get("name"), 40),
        };
    },
    behaviors: {
        boxFilmViewShow: kino.controller.boxFilmViewShow,
    },
    childViewTriggers: { // Proxy to main view
        "child:boxfilmview:show": "boxfilmview:show",
    },
});
kino.view.listLast = Marionette.CollectionView.extend({
    tagName: "div",
    className: "d-flex flex-wrap",
    childView: kino.view.itemLast,
    childViewTriggers: {  // Proxy to section view
        "boxfilmview:show": "child:boxfilmview:show",
    },
});
kino.view.sectionHome = Marionette.View.extend({
    template: _.template($('#tplSectionHome').html()),
    regions: {
        rLast: "#regionLast",
    },
    initialize() {
        this.listenTo(channel, 'all:reload', this.onReload);
    },
    templateContext() {
        return {
            "total": kino.model.sitedata.get("total"),
        };
    },
    behaviors: {
          boxFilmAddShow: kino.controller.boxFilmAddShow,
    },
    childViewTriggers: { // Proxy to main view
        "child:boxfilmview:show": "boxfilmview:show",
        "child:boxfilmadd:show": "boxfilmadd:show",
    },
    onRender() {
        var self = this;

        // Get data from server
        var ajx = Object.assign({}, ajaxDefaults);
        var defer = $.Deferred();
        kino.showLoading();
        this.model.fetch({
            timeout: JQX_TIMEOUT,
            cache: false,
            success: function(mdl){
                defer.resolve(mdl);
            },
            error: function(mdl, jqXHR, o){
                kino.ajaxErrorHandler(jqXHR, ajx, self.model.url);
                defer.resolve(undefined);
            },
        });
        $.when(defer.promise()).done(function(mdl){
            if (mdl !== undefined) { // OK
                // Render list: last
                self.showChildView('rLast', new kino.view.listLast({
                    collection: new FilmsCollection(mdl.get("last")),
                }));
            } else {
                kino.alert(ajx);
            }
            kino.hideLoading();
        });
    },
    onReload(msg){
        this.render();
    },
});

// Section: Search
kino.view.searchresultrow = Marionette.View.extend({
    tagName: "div", // "tr",
    className: "row mb-3", //  border-bottom bg-light
    template: _.template($('#tplSearchResultRow').html()),
    behaviors: {
        boxFilmViewShow: kino.controller.boxFilmViewShow,
        boxFilmAddShow: kino.controller.boxFilmAddShow,
    },
    ui: {
        btnFilmDel:  ".btnFilmDel",
    },
    triggers: {
        'click @ui.btnFilmDel':  'film:delete',
    },
    childViewTriggers: { // Proxy to main view
        "child:boxfilmview:show": "boxfilmview:show",
        "child:boxfilmadd:show": "boxfilmadd:show",
    },
});
kino.view.searchresult = Marionette.CollectionView.extend({
    tagName: "div", // table
    className: "container-fluid",
    childView: kino.view.searchresultrow,
    emptyView: kino.view.emptyDiv,
    emptyViewOptions: {
        message: "No any data",
        className: "text-secondary"
    },
    childViewTriggers: {  // Proxy to section view
        "boxfilmview:show": "child:boxfilmview:show",
        "boxfilmadd:show": "child:boxfilmadd:show",
        "film:delete": "child:film:delete",
    },
});
kino.view.searchform = Marionette.View.extend({
    template: _.template($('#tplSearchForm').html()),
    regions: {
        ctrlRegion:     "#ctrlSearchForm",
    },
    ui: {
        frmSearchForm:          "#searchForm",
    },
    events: {
        'submit @ui.frmSearchForm': 'submitForm',
    },
    modelEvents: {
        'change:storage': 'onChangeModel',
        'change:genre': 'onChangeModel',
        'change:name': 'onChangeModel',
    },
    onChangeModel(model, value) {
        //console.log("Модель изменилась");
        this.render();
    },
    templateContext() {
        var storage = this.model.get("storage");

        // Genre and Country lists
        var genres = [];
        var countries = [];
        if (_.isObject(kino.model.sitedata)) {
            genres = kino.model.sitedata.get("genres");
            countries = kino.model.sitedata.get("countries");
        }

        return {
            "storage": storage == 0 ? "" : storage,
            "genres": _.isEmpty(genres) ? [] : genres,
            "countries": _.isEmpty(countries) ? [] : countries,
        };
    },
    submitForm(event) {
        event.preventDefault();
        event.stopPropagation();

        // Data for searching
        var data = {};
        var sa = this.ui.frmSearchForm.serializeArray();
        $.each(sa, function(){
            data[this.name] = this.value
        });

        // Go to next (See Root form)
        this.trigger("form:submit", this, data);

        return false;
    },
    renderButton(loading) {
        if (loading) {// Rendering button
            this.showChildView('ctrlRegion', new kino.view.btnLoading("Searching..."));
        } else {
            this.showChildView('ctrlRegion', new kino.view.btnSend('<i class="fas fa-search text-light">&nbsp;</i> Search', "submit"));
        }
    },
    onRender() {
        this.renderButton(false);
    },
});
kino.view.paginationItem = Marionette.View.extend({
    tagName: "li",
    className: "page-item",
    template: _.template($('#tplPaginationItem').html()),
    onRender() {
        // Add classes
        if (this.model.get("is_disabled")) {
            this.$el.addClass("disabled");
        } else if (this.model.get("is_active")) { 
            this.$el.addClass("active");
        }
    },
    ui: {
        btnPage: ".btnPage",
    },
    events: {
        'click @ui.btnPage': 'onClickPage',
    },
    onClickPage(e){
        e.preventDefault();
        e.stopPropagation();

        var page = $(e.currentTarget).attr("data-page")*1;

        // Backward Proxy of event
        this.trigger("click:page", page * 1); // See next: serzik.view.commentsPaginationRoot

        return false;
    },
});
kino.view.paginationRoot = Marionette.CollectionView.extend({
    tagName: "ul",
    className: "pagination pagination-sm justify-content-end",
    childView: kino.view.paginationItem,
    childViewTriggers: {
        'click:page': 'click:page', // See next: kino.view.sectionSearch
    },
});
kino.view.sectionSearch = Marionette.View.extend({
    template: _.template($('#tplSectionSearch').html()),
    regions: {
        regionForm: "#regionSearchForm",
        regionResult: "#regionSearchResult",
        regionPagination: "#regionPagination",
    },
    model: kino.model.search,
    initialize() {
        this.listenTo(channel, 'search:byid', this.onSearchById);
        this.listenTo(channel, 'search:bydata', this.onSearchByData);
        this.listenTo(channel, 'all:reload', this.onReload);
        this.options.prev_data = {};
    },
    behaviors: {
          boxFilmAddShow: kino.controller.boxFilmAddShow,
    },
    childViewTriggers: { // Proxy to main view
        "child:boxfilmview:show": "boxfilmview:show",
        "child:boxfilmadd:show": "boxfilmadd:show",
    },
    childViewEvents: {
        "form:submit": "onSubmit", // С точки отправки формы
        "child:film:delete": "onFilmDelete",
        "click:page": "onClickPage",
    },
    onClickPage(page) { // Команда на перерисовку списка с пагинацией
        this.options.prev_data.page = page;
        this.onSubmit(null, this.options.prev_data);
        //console.log("Clic to " + page);
        //this.renderList(page);
    },
    onReload() {
        this.onSubmit(null, this.options.prev_data);
    },
    onSearchById(id) {
        //console.log("Вот в этой точке идёт принудительный запрос на сервер с ID: "+ id);

        // Create data for search and go-to onSubmit
        this.onSubmit(null, {"id": id});
    },
    onSearchByData(data) {
        this.onSubmit(null, data);
    },
    onSubmit(view, data) {
        var self = this;
        this.options.prev_data = data;
        //console.log(data);

        // Get data from server
        var ajx = Object.assign({}, ajaxDefaults);
        var defer = $.Deferred();
        if (view) view.renderButton(true); // Show loading button
        //kino.cleanAlert();
        kino.showLoading();
        this.model.fetch({
            timeout: JQX_TIMEOUT,
            cache: false,
            data: data,
            success: function(mdl){
                defer.resolve(mdl);
            },
            error: function(mdl, jqXHR, o){
                kino.ajaxErrorHandler(jqXHR, ajx, self.model.url);
                defer.resolve(undefined);
            },
        });
        $.when(defer.promise()).done(function(mdl){
            if (mdl !== undefined) { // OK
                self.renderResult();
                //console.log("Yuppi! :-D");
            } else {
                kino.alert(ajx);
            }
            if (view) view.renderButton(); // Hide loading button
            kino.hideLoading();
        });
    },
    onFilmDelete(view) {
        var self = this;
        if (confirm("Are you sure you want to delete the film?") == false) return false;

        // Get data from server
        var ajx = Object.assign({}, ajaxDefaults);
        var defer = $.Deferred();
        kino.cleanAlert();
        kino.showLoading();

        // Delete on server
        var curstat = view.model.destroy({
            url: JQX_PFX_URL + "/film/" + view.model.id,
            wait: true,
            timeout: JQX_TIMEOUT,
            success: function(mdl){
                defer.resolve(mdl);
            },
            error: function(mdl, jqXHR, o){
                kino.ajaxErrorHandler(jqXHR, ajx, self.model.url);
                defer.resolve(undefined);
            },
        });
        $.when(defer.promise()).done(function(mdl){
            if (mdl !== undefined) { // OK
                //self.renderResult();
                //console.log("Yuppi! :-D");
            } else {
                kino.alert(ajx);
            }
            kino.hideLoading();
        });
    },
    onRender() {
        var self = this;

        // Render search form
        this.showChildView('regionForm', new kino.view.searchform({
            model: this.model,
            //model: ccpaygw.model.searchform
        }));
    },
    renderResult() {
        // Render result table
        var result = this.model.get("films");
        if (! _.isArray(result)) result = [];
        if (_.isEmpty(result)) {
            this.showChildView('regionResult', new kino.view.emptyDiv({
               message: "No data found",
            }));
        } else {
            this.showChildView('regionResult', new kino.view.searchresult({
               collection: new FilmsCollection(result),
            }));
            //$(window).scrollTop($(this.getRegion("regionResult").el).position().top - 100);
            $('.lightzoom').lightzoom({
                viewTitle: true,
                isOverlayClickClosing: true,
                isWindowClickClosing: true,
                isEscClosing: true,
            });

            //console.log(this.model);
            // Show pagination
            var total = this.model.get("total") || 0;
            var page = this.model.get("page") || 1;
            var pages = this.model.get("pages") || 0;
            if (total) {
                kino.collection.pagination = new PaginationCollection(
                        //getSimplePaginationStruct(page, pages)
                        getPaginationStruct(page, pages)
                    );
                var paginationRoot = new kino.view.paginationRoot({
                    collection: kino.collection.pagination,
                });
                this.showChildView('regionPagination', paginationRoot);
            }

        }
    },
});

// Root Application Views
kino.view.about = Marionette.View.extend({
    tagName:  "div",
    template: _.template($('#tplAboutApp').html()),
    model: kino.model.sitedata,
    templateContext() {
        return {
            "message": this.model.get("message") ? this.model.get("message") : kino.error_message || "unknown",
            //"date": this.model.get("published") ? dtf_short(this.model.get("published"), false) : "-",
            //"published": this.model.get("published") ? dtf_short(this.model.get("published"), true) : "-",
        };
    },
    renderResult() {
        var self = this;
        this.render();
    },
    onMetaReady(msg) {
        this.renderResult();
    },
});
kino.view.main = Marionette.View.extend({
    tagName:  "div",
    template: _.template($('#tplMainApp').html()),
    regions: {
        regionSectionHome: "#home",
        regionSectionSearch: "#search",
        regionSectionFeedback: "#feedback",
        regionSectionNoRFC: "#norfc",
        rBoxFilmView: "#regionBoxFilmView",
        rBoxFilmAdd: "#regionBoxFilmAdd",
    },
    initialize() {
        this.listenTo(channel, 'boxfilmadd:show', this.onBoxFilmAddShow);
    },
    onRender() {
        //console.log("Рендеримся");
        this.renderResult();
    },
    renderResult() {
        var self = this;

        // Render Section Home
        this.showChildView('regionSectionHome', new kino.view.sectionHome({
            model: kino.model.films,
        }));

        // Render Section Search
        this.showChildView('regionSectionSearch', new kino.view.sectionSearch());

        // Render BoxFilmView
        this.showChildView('rBoxFilmView', new kino.view.boxFilmView({
            model: kino.model.film,
        }));

        // Render BoxFilmAdd
        this.showChildView('rBoxFilmAdd', new kino.view.boxFilmAdd({
            model: kino.model.film,
        }));

    },
    childViewEvents: {
        'boxfilmview:show': 'onBoxFilmViewShow',
        'boxfilmadd:show': 'onBoxFilmAddShow',
        'filmadd:submit': 'onFilmAddSubmit',
        'searchbyid:click': 'onSearchByIDClick',
    },
    onBoxFilmViewShow(mdl) {
        //console.log("Wow! Catched this event #" + id||0);
        //this.trigger("child:hostname:change", v.model.get("hostname")); // Go to next
        this.getChildView('rBoxFilmView').showModal(mdl);
    },
    onBoxFilmAddShow(mdl) {
        //console.log("Wow! Catched this event #" + id||0);
        //this.trigger("child:hostname:change", v.model.get("hostname")); // Go to next
        this.getChildView('rBoxFilmAdd').showModal(mdl);
    },
    onFilmAddSubmit(view, formdata) {
        var self = this;

        // Send data to server
        var ajx = Object.assign({}, ajaxDefaults);
        var defer = $.Deferred();
        kino.showLoading();

        // Upload to server
        var curstat = $.ajax({
            url: JQX_PFX_URL+"/film",
            type: 'POST',
            data: formdata,
            cache: false,
            dataType: 'json',
            processData: false,
            contentType: false, 
            success: function(mdl){
                defer.resolve(mdl);
            },
            error: function(mdl, jqXHR, o){
                kino.ajaxErrorHandler(mdl, ajx, JQX_PFX_URL+"/film");
                defer.resolve(undefined);
            },
            xhr: function() {
                var xhr = new window.XMLHttpRequest();
                xhr.upload.addEventListener('progress', function(e) {
                    if (e.lengthComputable) {
                        var val = Math.floor(100 * e.loaded / e.total);
                        view.progress(val);
                    }
                });
                return xhr;
            },
        });
        if (curstat) {
            $.when(defer.promise()).done(function(mdl){
                kino.hideLoading();
                if (mdl !== undefined) { // OK
                    if (mdl.storage_last && mdl.storage_last > kino.model.sitedata.get("storage_last")) {
                        kino.model.sitedata.set("storage_last", mdl.storage_last);
                        //console.log("Added! "+ kino.model.sitedata.get("storage_last"));
                    }
                    
                    //self.renderList();
                    kino.pushAlert({
                        noAdd: true,
                        level: "info",
                        code: 200,
                        title: "Ok",
                        message: "Film has been successfully saved",
                    });
                } else {
                    kino.alert(ajx);
                }
                view.hideModal();
                channel.trigger("all:reload", "Run channel trigger all:reload after send data");
            });
        } else {
            view.hideModal();
            kino.hideLoading();
            kino.pushAlert({
                noAdd: true,
                level: "danger",
                code: 400,
                title: "Request incorrect",
                message: "Request incorrect",
            });
        };

        return true;
    },
    onSearchByIDClick(view, id) {
        var self = this;
        view.hideModal();
        channel.trigger("search:byid", id);
        return true;
    },
});

/*
    Applications
*/
const AboutApp = Marionette.Application.extend({ // About
    region: "#AboutApp", // #AboutApp
    onStart() {
        this.aboutView = new kino.view.about();
        this.showView(this.aboutView);
    },
    initialize() {
        this.options.isInitialized = false;
        this.listenTo(channel, 'all:ready', this.onReady);
    },
    onReady(msg) {
        if (this.options.isInitialized) return true;
        if (kino.model.sitedata.get("version")) {
            // Render about databox
            this.aboutView.triggerMethod('meta:ready', msg);
            this.options.isInitialized = true;
        }
    },
});
kino.app.about = new AboutApp();
kino.app.about.start();

const MainApp = Marionette.Application.extend({ // Main
    region: "#MainApp",
    onStart() {
        this.mainView = new kino.view.main();
    },
    initialize() {
        this.listenTo(channel, 'all:ready', this.onReady); // For initialize
        //this.listenTo(channel, 'all:reload', this.onReload); // For reloading application data
        this.listenTo(channel, 'search:byid', this.onSearchById); // Вот это нужно повторить на самом саб-вью Search для поиска по ID
        this.listenTo(channel, 'search:bydata', this.onSearchByData); // For search by storage, genre, id and name
    },
    onReady(msg) { // Initialize application
        this.showView(this.mainView);
        //console.log(msg);
        //this.loadSummaryData();
    },
    onSearchById(id) {
        // Show search tab only. See search view
        $("#search-tab").tab('show');
    },
    onSearchByData(data) {
        // Show search tab only. See search view
        $("#search-tab").tab('show');
    }
});
kino.app.main = new MainApp();
kino.app.main.start();

kino.run();

/* __END__ */