/*  $Id: main.js 16 2020-09-21 22:27:15Z minus $
 *
 *  main.js
 *  Version 1.00 2021
 *  Kinoteka: global definitions and common utilities
 *                                                       ____________
 *  Copyright (C) 1998-2021 D&D Corporation.             \          /
 *  All rights reserved.                                  \________/
 *  Serż Minus (Sergey Lepenkov), http://www.serzik.com    \      /
 *                                                          \____/
 *  Dual licensed under the MIT and GNU GPL v3. licenses:    \  /
 *   http://www.opensource.org/licenses/mit-license.php       \/
 *   http://www.gnu.org/licenses/gpl.html
 *
*/

$(document).ready(function(){

    // Home link
    $('#lnkHome').on('click', function (e) {
        e.preventDefault();
        $("#home-tab").tab('show');
    })

    // Search from navbar
    $('#frmSearchNav').on('submit', function (e) {
        e.preventDefault();
        e.stopPropagation();

        var sa = $(e.currentTarget).serializeArray();
        var data = {};
        $.each(sa, function(){
            data[this.name] = this.value;
        });
        if (_.isEmpty(data.search)) return false;
        var search = data.search.trim();
        var firstch = search.substring(0, 1);
        var lastchs = search.substring(1);
        if (firstch == "!") { // ID
            channel.trigger("search:byid", lastchs);
        } else if (firstch == "#") { // Storage
            channel.trigger("search:bydata", {"storage": lastchs});
        } else if (firstch == "@") { // Genre
            channel.trigger("search:bydata", {"genre": lastchs});
        } else { // NAme
            channel.trigger("search:bydata", {"name": search});
        }

        $(e.currentTarget).trigger("reset");
        return false;
    })

    // Magic button
    /*
    $('#btnMagic').on('click', function (e) {
        channel.trigger("boxfilmadd:show", null);
        return false;
    });
    */

    /* Tooltips */
    $( ".tooltip" ).tooltip();
    $('[data-toggle="tooltip"]').tooltip();
});

/*
    Constants
*/

const PROJECT_NAME = "Kinoteka";
const PROJECT_PREFIX = "kinoteka";
const PROJECT_VERSION = "0.00";
const JQX_TIMEOUT = 10*1000; // 10 secs
const JQX_TIMEOUT_META = 500; // 0.5 secs
const JQX_PFX_URL = "api";
const SITEDATA_URL = "api"; // sitedata.json
const ajaxDefaults = { // For Ajax statuses
    status_code: 200,
    status_text: "OK",
    code: 0,
    message: "OK",
};

/*
    Utilities
*/

function timeConverter(UNIX_timestamp){
    var a = new Date(UNIX_timestamp * 1000);
    var months = ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'];
    var year = a.getFullYear();
    var month = months[a.getMonth()];
    var date = a.getDate();
    var hour = ('0' + a.getHours()).slice(-2);
    var min = ('0' + a.getMinutes()).slice(-2);
    var sec = ('0' + a.getSeconds()).slice(-2)
    var time = date + ' ' + month + ' ' + year + ' ' + hour + ':' + min + ':' + sec ;
    return time;
}
function dtf(u) { // Unixtime
    var dt = u ? (new Date(u * 1000)) : new Date();
    var year    = dt.getFullYear();
    var month   = ('0' + (dt.getMonth() + 1)).slice(-2);
    var day     = ('0' + dt.getDate()).slice(-2);
    var hour    = ('0' + dt.getHours()).slice(-2);
    var min     = ('0' + dt.getMinutes()).slice(-2);
    //var sec     = ('0' + dt.getSeconds()).slice(-2)
    return day + '.' + month + '.' + year + ' ' + hour + ':' + min; //  + ':' + sec
}
function dtf_short(u, t) { // Unixtime
    var dt = u ? (new Date(u * 1000)) : new Date();
    var year    = dt.getFullYear();
    var month   = ('0' + (dt.getMonth() + 1)).slice(-2);
    var day     = ('0' + dt.getDate()).slice(-2);
    var hour    = ('0' + dt.getHours()).slice(-2);
    var min     = ('0' + dt.getMinutes()).slice(-2);
    var sec     = ('0' + dt.getSeconds()).slice(-2)
    if (t) {
        return year + '-' + month + '-' + day + ' ' + hour + ':' + min; //  + ':' + sec
    }
    return year + '-' + month + '-' + day;        
}
function fduration(msecs){
    var secs = Math.trunc((msecs || 0)*1);
    var years = Math.trunc(secs / (60*60*24*365));
    secs -= years * 60*60*24*365;
    var days = Math.trunc(secs / (60*60*24));
    secs -= days * 60*60*24;
    var hours = Math.trunc(secs / (60*60));
    secs -= hours * 60*60;
    var mins = Math.trunc(secs / 60);
    secs = secs % 60;

    if (years) {
        return years + " years "+ days +" days " + hours + " hours";
    } else if (days) {
        return days + " days " + hours + " hours " + mins + " minutes";
    } else if (hours) {
        return hours + " hours " + mins + " minutes " + secs + " seconds";
    } else if (mins >= 2) {
        return mins + " minutes " + secs + " seconds";
    } else if (secs > 5) {
        secs += mins * 60;
        return secs + " seconds";
    } else if (msecs - secs) {
        return msecs + " seconds";
    } else {
        return secs + " seconds";
    }
}
function digsf(inv){
    invs = inv + "";
    return invs.replace(/(\d)(?=(\d\d\d)+([^\d]|$))/g, "$1`");
}
function formatBytes(bytes,decimals) {
   if(bytes == 0) return '0 Bytes';
   var k = 1024,
       dm = decimals <= 0 ? 0 : decimals || 2,
       sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'],
       i = Math.floor(Math.log(bytes) / Math.log(k));
   return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
}
function compressIPV6(input) {
    var formatted = input.replace(/\b(?:0+:){2,}/, ':');
    var finalAddress = formatted.split(':')
        .map(function(octet) {
            return  octet.replace(/\b0+/g, '');
        }).join(':');
    if (finalAddress === ":") return "::";
    return finalAddress;
}
function getBIT(v,n){
    if (!v) v = 0;
    if (!n) n = 0;
    return (v & (1 << n)) ? 1 : 0;
}
function setBIT(v,n){
    if (!v) v = 0;
    if (!n) n = 0;
    return (v | Math.pow(2,n));
}
function unsetBIT(v,n){
    if (!v) v = 0;
    if (!n) n = 0;
    return (v ^ Math.pow(2,n));
}
function validateEmail(email) {
    var reg = /^\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$/;
    if (reg.test(email)) {
        return true;
    }
    return false;
}
function statusf(i) {
    if (_.isNumber(i) && !_.isNaN(i)) {
        if (i == -1) {
            return "warning";
        } else if (i == 0) {
            return "critical";
        } else if (i == 1) {
            return "good";
        }
    }
    return "unknown";
}
function cutLongString(text, count_lit){
    var all_len = text.length;
    if (all_len > count_lit){
        return text.substr(0, (count_lit - 1)) + '…';
    }
    return text;
}
function getSimplePaginationStruct(page, tabs) {
    var struct = [];
    page=page*1;
    //if (!count) return struct;

    // Подсчет количества табов
    //var tabs = ((count - (count % COMMENTS_PER_PAGE)) / COMMENTS_PER_PAGE) + ((count % COMMENTS_PER_PAGE) ? 1 : 0);
    if (tabs <= 1) return struct;

    /*
    <li class="page-item disabled">
      <a class="page-link" href="#" aria-label="Previous" tabindex="-1" aria-disabled="true"><span aria-hidden="true">&laquo;</span></a>
    </li>
    <li class="page-item active"><a class="page-link" href="#">1 <span class="sr-only">(current)</span></a></li>
    <li class="page-item"><a class="page-link" href="#">2</a></li>
    <li class="page-item"><a class="page-link" href="#">3</a></li>
    <li class="page-item">
      <a class="page-link" href="#" aria-label="Next"><span aria-hidden="true">&raquo;</span></a>
    </li>
    */

    // Рисуем "назад"
    struct.push({
        page: page>1 ? page - 1 : 1,
        title: '&laquo;',
        label: 'Previous',
        is_disabled: page>1 ? 0 : 1,
    });

    // Рисуем сам таб
    for (i = 1; i <= tabs; i++) {
        struct.push({
            page: i,
            title: i,
            is_active: (page == i) ? 1 : 0,
        });
    }

    // Рисуем "вперед"
    struct.push({
        page: page + 1,
        title: '&raquo;',
        label: 'Next',
        is_disabled: (page >= tabs) ? 1 : 0,
    });

    return struct;
}
function getPaginationStruct(page, pages) {
    const btn_max = 10; // Кнопки (квадратики, табики)
    const win_max = 10; // Окно (блоки кнопок)

    var struct = [];
    page = page*1;
    if (pages <= 1) return struct;

    // Рисуем "назад"
    struct.push({
        page: page > 1 ? page - 1 : 1,
        title: '&laquo;',
        label: 'Previous',
        is_disabled: page > 1 ? 0 : 1,
    });

    // Определяем начальную и конечную позицию окна
    var win_start = page - (win_max/2);
    var delta_start = win_start<=0 ? win_start * -1 : 0;
    var win_finish = page + (win_max/2);
    var delta_finish = win_finish>=pages ? pages - win_finish + 1 : 0;

    // Рисуем начало
    if (page > win_max) {
        struct.push({
            page: 1,
            title: 1,
        });        
    }

    // Рисуем точечки влево
    if (page > win_max) {
        struct.push({
            page: page - win_max - 1,
            title: "...",
        });
    }

    // Рисуем саму кнопку
    for (i = 1; i <= pages; i++) {
        if ((i > (win_start+delta_finish)) && (i < (win_finish+delta_start))) {
            struct.push({
                page: i,
                title: i,
                is_active: (page == i) ? 1 : 0,
            });
        }
    }

    // Рисуем точечки вправо
    if (page < (pages-win_max)) {
        struct.push({
            page: page + win_max - 1,
            title: "...",
        });
    }

    // Рисуем конец
    if (page < (pages-win_max)) {
        struct.push({
            page: pages,
            title: pages,
        });        
    }

    // Рисуем "вперед"
    struct.push({
        page: page + 1,
        title: '&raquo;',
        label: 'Next',
        is_disabled: (page >= pages) ? 1 : 0,
    });

    return struct;
}


/* __END__ */
